<?php

/*
 * AfterLogic Admin Panel by AfterLogic Corp. <support@afterlogic.com>
 *
 * Copyright (C) 2002-2009  AfterLogic Corp. (www.afterlogic.com)
 * Distributed under the terms of the license described in LICENSE.txt
 * 
 */

class CWmServerConsole
{

	var $_admHost;
	var $_admPort;
	var $_admLogin;
	var $_admPassword;
	
	var $_socket;
	var $_socket_status;
	var $_error;
	var $_clear_error;
	
	var $_settings;
	
	var $_userVars = array(	'RealName', 
							'HomePage',
							'Address',
							'Telephone',
							'MaxMBSize',
							'SmtpPerms',
							'ReceiveEnable',
							'PopEnable');
	
	/**
	 * @param string $admLogin
	 * @param string $admPassword
	 * @param string $admHost
	 * @param string $admPort
	 * @return CWmServerConsole
	 */
	function CWmServerConsole($admLogin, $admPassword, $admHost, $admPort)
	{	
		$this->_admHost = $admHost;
		$this->_admPort = $admPort;
		$this->_admLogin = $admLogin;
		$this->_admPassword = $admPassword;
	}

	/**
	 * @return bool
	 */
	function Connect()
	{
		$errstr = '';
		$errno = 0;
		$timeout = 3;
		
		$this->_socket = @fsockopen($this->_admHost, $this->_admPort, $errno, $errstr, $timeout);
		if(!$this->_socket)
		{
			$this->_error = 'Connect() - Error: Can\'t connect to XMail Server';
			$this->_clear_error = 'Can\'t connect to XMail Server';
			return false;
		}

		@socket_set_timeout($this->_socket, 10);
		
		$response = $this->_readline();
		if (substr($response, 0, 1) == '-' )
		{
			$this->_error = 'Connect() - Error: '.$response;
			$this->_clear_error = $response;
			$this->_cleanup();
			return false;
		}	

        @socket_set_blocking($this->_socket, true);
        		
		return $this->Login();
	}

	/**
	 * @return bool
	 */
	function Login()
	{
		$this->_write($this->_admLogin."\t".$this->_admPassword);
		$response = $this->_readline();
		if (substr($response,0,1) == '-' )
		{
			$this->_error = 'Wrong login and/or password. Authentication failed.';
			$this->_clear_error = 'Wrong login and/or password. Authentication failed.';
			$this->_cleanup();
			return false;
		}		
		
		return true;
	}
	
	/**
	 * @access public
	 * @return bool
	 */
	function Disconnect()
	{
		if ($this->_socket == false)
		{
			return true;
		}
		$this->Logout();
		@fclose($this->_socket);
		$this->_socket = false;
		return true;
	}
	
	/**
	 * @access public
	 * @return bool
	 */
	function Logout()
	{
		$this->_write('quit');
		return $this->_checkResponse($this->_readline(), 'Logout()', __LINE__);
	}

	/*==================Domains====================*/
	/**
	 * @param string $domain
	 * @return bool
	 */
	function AddDomain($newDomain)
	{
		//"domainadd"[TAB]"domain"
		$this->_write("domainadd\t$newDomain");
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'AddDomain()');
	}
	
	/**
	 * @param string $domain
	 * @return bool
	 */
	function DeleteDomain($domain)
	{
		/* "domaindel"[TAB]"domain" */
		$this->_write("domaindel\t$domain");
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'DeleteDomain()');
	}
	
	/**
	 * @param string $domain
	 * @return bool
	 */
	function DomainExist($domain)
	{
		$domailArray = $this->DomainList();
		return in_array($domain, $domailArray);
	}
	
	/**
	 * @return array
	 */
	function DomainList()
	{
		/* "domainlist"*/
		$domainsArray = array();
		$this->_write("domainlist");
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'DomainList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$domainsArray[] = trim($response, '"');
				}
				else break;			
			} while ($response != '.');
		}
		
		return $domainsArray;
	}

	/**
	 * @param string $newDomain
	 * @return bool
	 */
	function AddCustomDomain($newDomain)
	{
		//"custdomset"[TAB]"domain"<CR><LF>
		$this->_write("custdomset\t$newDomain");
		$response = $this->_readline();
		
		if ($this->_checkResponse($response, 'AddCustomDomain();')) 
		{
			$this->_write("\r\n.");
			$response = $this->_readline();
			
			return $this->_checkResponse($response, 'AddCustomDomain();');
		} else {
			return false;
		}
	}
	
	/**
	 * @param string $domain
	 * @return bool
	 */
	function DeleteCustomDomain($domain)
	{
		/* "custdomdel"[TAB]"domain"<CR><LF> */
		$this->_write("custdomdel\t$domain");
		$response = $this->_readline();
		return $this->_checkResponse($response, 'DeleteCustomDomain();');
	}
	
	/**
	 * @param string $domain
	 * @return bool
	 */
	function CustomDomainExist($domain)
	{
		$domainArray = $this->CustomDomainList();
		return in_array($domain, $domainArray);
	}
	
	/**
	 * @return array
	 */
	function CustomDomainList()
	{
		//"custdomainlist"
		$customDomainsArray = array();
		$this->_write("custdomlist");
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'CustomDomainList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$customDomainsArray[] = trim($response, '"');
				}
				else break;			
			} while ($response != '.');
		}
		
		return $customDomainsArray;
	}
	
	//=====================================Users===============================================================
	/**
	 * @return array
	 */
	function UserList($domainname, $username = null, $start = null, $usrcount = null)
	{
		//"userlist"[TAB]"domain"[TAB]"username"[TAB]"start_number"[TAB]"request_amount"<CR><LF>
		$usersarray = array();
		$username = ($username === '' || $username === null) ? '*' : $username;
		if ($domainname !== null && $username !== null && $start !== null && $usrcount !== null)
		{
			$this->_write("userlist\t$domainname\t$username\t$start\t$usrcount");	
		} 
		elseif ($domainname !== null && $username !== null) 
		{
			$this->_write("userlist\t$domainname\t$username");	
		}
		else
		{
			$this->_write("userlist\t$domainname");
		}

		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'UserList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tempUserArray = explode("\t", $response);
					$usersarray[] = array_map('myTrim', $tempUserArray);
				}
				else break;			
			} while ($response != '.');
		}
		
		return $usersarray;		
	}
	
	/**
	 *
	 * @param string $domain
	 * @param string $search[optional] = ''
	 * @return int
	 */
	function GetUsersCount($domain, $search = '') 
	{
		//"usercount"[TAB]"domain"<CR><LF>
		$usrCount = 0;
		if (strlen($search) > 0)
		{
			$this->_write("usercount\t".$domain."\t".$search);	
		}
		else
		{
			$this->_write("usercount\t".$domain);	
		}
			
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'GetUsersCount()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tempUserArray = explode("\t", $response);
					$tempArray = array_map('myTrim', $tempUserArray);
					if (count($tempArray) == 2 && $tempArray[0] == $domain)
					{
						$usrCount = $tempArray[1];
					}
				}
				else break;			
			} while ($response != '.');
		} 
		return $usrCount;
	}
	
	/**
	 * @param $domain
	 * @param $username
	 * @return bool
	 */
	function IsMultiListUser($domain, $username) 
	{
		$userlist = $this->UserList($domain, $username);
		return (count($userlist) > 0 && $userlist[0][3] == 'M'); 
	}
	
	/**
	 * @param string $domain
	 * @param string $username
	 * @param int $newSize
	 * @return bool
	 */
	function ChangeUserMaxMailBoxSize($domain, $username, $newSize)
	{
		if ($this->UserExist($domain, $username))
		{
			//"uservarsset"[TAB]"domain"[TAB]"username"[TAB]"varname"[TAB]"varvalue" ... <CR><LF>
			$this->_write("uservarsset\t$domain\t$username\tMaxMBSize\t$newSize");
			$response = $this->_readline();
			return $this->_checkResponse($response, 'ChangeUserMaxMailBoxSize()');
		}
		else 
		{
			return false;
		}
	}
	
	/**
	 * @param $domain
	 * @param $username
	 * @param string $realname
	 * @param string $homepage
	 * @param int $mbsize
	 * @param '1' or '0' $storelocaly
	 * @param '1' or '0' $closedml
	 * @return bool
	 *
	function ChangeUserVars($domain, $username, $realname, $homepage, $mbsize, $storelocaly, $closedml) 
	{
		if ($this->UserExist($domain, $username))	
		{
			//"uservarsset"[TAB]"domain"[TAB]"username"[TAB]"varname"[TAB]"varvalue" ... <CR><LF>
			if ($realname == '') $realname = ' ';
			if ($homepage == '') $homepage = ' ';
			$this->_write("uservarsset\t$domain\t$username\tRealName\t$realname\tHomePage\t$homepage\tMaxMBSize\t$mbsize\tStoreMailLocaly\t$storelocaly\tClosedMailList\t$closedml");
			$response = $this->_readline();
			return $this->_checkResponse($response, 'ChangeUserVars()');	
		}
		else 
		{
			return false;
		}
	}*/
	
	/**
	 * @param $domain
	 * @param $username
	 * @param string $realname
	 * @param string $homepage
	 * @param int $mbsize
	 * @param '1' or '0' $storelocaly
	 * @param '1' or '0' $closedml
	 * @return bool
	 */
	function ChangeUserVars($domain, $username, $realname, $homepage, $mbsize, $storelocaly, $closedml) 
	{
		//"uservarsset"[TAB]"domain"[TAB]"username"[TAB]"varname"[TAB]"varvalue" ... <CR><LF>
		if ($realname === '' || $realname === null) $realname = ' ';
		if ($homepage === '' || $homepage === null) $homepage = ' ';
		$this->_write("uservarsset\t$domain\t$username\tRealName\t$realname\tHomePage\t$homepage\tMaxMBSize\t$mbsize\tStoreMailLocaly\t$storelocaly\tClosedMailList\t$closedml");
		return $this->_checkResponse($this->_readline(), 'ChangeUserVars()');	
	}

	/**
	 * @param $domain
	 * @param $username
	 * @param string $realname[optional] = null
	 * @param int $mbsize[optional] = null
	 * @param bool $storelocaly[optional] = null
	 * @return bool
	 */
	function ChangeUserVarsNew($domain, $username, $realname = null, $mbsize = null, $storelocaly = null) 
	{
		$cmd = "uservarsset\t$domain\t$username";
		if ($realname !== null || $mbsize !== null || $storelocaly !== null)
		{
			if ($realname.$realname !== null && strlen($realname) > 0)
			{
				$cmd .= "\tRealName\t$realname";
			}
			if ($mbsize !== null && strlen($mbsize) > 0)
			{
				$cmd .= "\tMaxMBSize\t$mbsize";
			}
			if ($storelocaly !== null)
			{
				$cmd .= "\tStoreMailLocaly\t". ((int) $storelocaly);
			}
			$this->_write($cmd);
			return $this->_checkResponse($this->_readline(), 'ChangeUserVarsNew()');
		}
		
		return true;
	}
	
	/**
	 * @param $domain
	 * @param $username
	 * @return array
	 *
	function GetUserVars($domain, $username) 
	{
		//uservars"[TAB]"domain"[TAB]"username"<CR><LF>
		$varArray = array();
		if ($this->UserExist($domain, $username))	
		{
			$this->_write("uservars\t".$domain."\t".$username);	
			$resp = trim($this->_readline());
			if ($this->_checkResponse($resp, 'GetUserVars()'))
			{
				do 
				{
					$response = trim($this->_readline());
					if ($response != '.')
					{
						$tempVarArray = explode("\t", $response);
						$varArray[myTrim($tempVarArray[0])] = myTrim($tempVarArray[1]);
					}
					else break;			
				} while ($response != '.');
			}
			return $varArray;	
		} 
		else 
		{
			return false;
		}
	}*/
	
	/**
	 * @param $domain
	 * @param $username
	 * @return array
	 */
	function GetUserVars($domain, $username) 
	{
		//uservars"[TAB]"domain"[TAB]"username"<CR><LF>
		$varArray = array();
		
		$this->_write("uservars\t".$domain."\t".$username);	
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'GetUserVars()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tempVarArray = explode("\t", $response);
					$varArray[myTrim($tempVarArray[0])] = myTrim($tempVarArray[1]);
				}
				else break;			
			} while ($response != '.');
		}
		return $varArray;	
	}
	
	/**
	 * @param $domain
	 * @param $username
	 * @return bool
	 */
	function UserExist($domain, $username)
	{
		$usersArray = $this->UserList($domain, $username);
		return (count($usersArray) > 0);
	}
	
	/**
	 * @param $domain
	 * @param $username
	 * @param $password
	 * @param 'U' or 'M' $usertype
	 * @return bool
	 */
	function AddUser($domain, $username, $password, $usertype = 'U')
	{
		//"useradd"[TAB]"domain"[TAB]"username"[TAB]"password"[TAB]"usertype"
		$this->_write("useradd\t".$domain."\t".$username."\t".$password."\t".$usertype);
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'AddUser()');		
	}
	
	/**
	 * @param string $domain
	 * @param string $username
	 * @return bool
	 */
	function DeleteUser($domain, $username)
	{
		//"userdel"[TAB]"domain"[TAB]"username"
		$this->_write("userdel\t".$domain."\t".$username);
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'DeleteUser()');		
	}
	
	/**
	 * @param string $domain
	 * @param string $username
	 * @param string $newPassword
	 * @return bool
	 */
	function ChangeUserPass($domain, $username, $newPassword)
	{
		//"userpasswd"[TAB]"domain"[TAB]"username"[TAB]"password"
		$this->_write("userpasswd\t".$domain."\t".$username."\t".$newPassword);
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'ChangeUserPass()');			
	}

	/**
	 * @param string $domain
	 * @param string $username
	 * @param string $adress
	 * @param 'R'|'RW' $perms
	 * @return bool
	 */
	function AddMlUser($domain, $username, $adress, $perms = null)
	{
		//"mluseradd"[TAB]"domain"[TAB]"mlusername"[TAB]"mailaddress"[TAB]"perms"<CR><LF>
		if ($perms === null)
		{
			$this->_write("mluseradd\t".$domain."\t".$username."\t".$adress);
		}
		else
		{
			$this->_write("mluseradd\t".$domain."\t".$username."\t".$adress."\t".$perms);
		}
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'AddMlUser()');	
	}
	
	/**
	 * @return bool
	 */
	function DeleteMlUser($domain, $username, $adress)
	{
		//"mluserdel"[TAB]"domain"[TAB]"mlusername"[TAB]"mailaddress"<CR><LF>
		$this->_write("mluserdel\t".$domain."\t".$username."\t".$adress);
		return $this->_checkResponse($this->_readline(), 'DeleteMlUser()');	
	}
	
	/**
	 * @return array
	 */
	function MlUserList($domain, $username)
	{
		//"mluserlist"[TAB]"domain"[TAB]"mlusername"<CR><LF>
		$mlusersArray = array();
		$this->_write("mluserlist\t".$domain."\t".$username);
		
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'MlUserList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tmpArray = explode("\t", $response);
					$mlusersArray[] = array_map('myTrim', $tmpArray);
				}
				else break;			
			} while ($response != '.');
		}
		return $mlusersArray;	
	}
	
	/**
	 * @return bool
	 */
	function MlUserExist($domain, $username, $mluseradress) 
	{
		$mluserList = array();
		$tmpList = $this->MlUserList($domain, $username);
		foreach ($tmpList as $value) {
				$mluserList[] = $value[0];
		}
		return in_array($mluseradress, $mluserList);		
	}
	
	/**
	 * @param string $domain
	 * @param string $username
	 * @return bool
	 */
	function getUserPassword($domain, $username) 
	{
		//cfgfileget[tab]path
		$tab = array();
		if ($this->UserExist($domain, $username))	
		{
			$path = "domains/$domain/mailusers.tab";
			$this->_write("cfgfileget\t$path");	
			$resp = trim($this->_readline());
			if ($this->_checkResponse($resp, 'getUserPassword()'))
			{
				do 
				{
					$response = trim($this->_readline());
					if ($response != '.')
					{
						$tempVarArray = explode("\t", $response);
						$tab[myTrim($tempVarArray[1])] = myTrim($tempVarArray[2]);
					}
					else break;			
				} while ($response != '.');
			}
			return DecryptPassword($tab[$username]);	
		} 
		else 
		{
			return false;
		}	
	}
	//=====================================Aliases=============================================================
	/**
	 * @param string $domain
	 * @param string $username
	 * @param string $alias
	 * @return bool
	 */
	function AddAlias($domain, $username, $alias) 
	{
		//"aliasadd"[TAB]"domain"[TAB]"alias"[TAB]"account"<CR><LF>
		$this->_write("aliasadd\t".$domain."\t".$alias."\t".$username);
		return $this->_checkResponse($this->_readline(), 'AddAlias()');
	}
	
	/**
	 * @param string $domain
	 * @param string $alias
	 * @return bool
	 */
	function DeleteAlias($domain, $alias) 
	{
		//"aliasdel"[TAB]"domain"[TAB]"alias"<CR><LF>
		$this->_write("aliasdel\t".$domain."\t".$alias);
		return $this->_checkResponse($this->_readline(), 'AddAlias()');
	}
	
	/**
	 * @return array
	 */
	function AliasList($domainname, $alias = null, $username = null)
	{
		//"aliaslist"[TAB]"domain"[TAB]"alias"[TAB]"account"<CR><LF>
		$aliasarray = array();
		if ($domainname && $alias && $username)
		{
			$this->_write("aliaslist\t".$domainname."\t".$alias."\t".$username);	
		}
		else if ($domainname && $alias && $username == null)
		{
			$this->_write("aliaslist\t".$domainname."\t".$alias);
		}
		else
		{
			$this->_write("aliaslist\t".$domainname);
		} 
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'AliasList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tempArray = explode("\t", $response);
					$aliasarray[] = array_map('myTrim', $tempArray);
				}
				else break;			
			} while ($response != '.');
		}
		
		return $aliasarray;		
	}
	
	/**
	 * @param string $domain
	 * @param string $username
	 *
	function UserAliasList($domain, $username)
	{
		$alias = array();
		$aliasList = $this->AliasList($domain);
		foreach ($aliasList as $value) {
			if ($value[2] == $username) {
				$alias[] = $value[1];
			}
		}
		return $alias;
	}*/
	
	/**
	 * @param string $domain
	 * @param string $username
	 */
	function UserAliasList($domain, $username)
	{
		return $this->AliasList($domain, '*', $username);;
	}
	
	/**
	 * @param string $domain
	 * @param string $alias
	 * @return unknown
	 */
	function AliasExist($domain, $alias)
	{
		$aliasArray = array();
		$tmpAliasArray = $this->AliasList($domain);
		foreach ($tmpAliasArray as $value) {
				$aliasArray[] = $value[1];
		}
		return in_array($alias, $aliasArray);
	}
	
	//=====================================External Links======================================================
	
	/**
	 * @return bool
	 */
	function AddExtLink($domain, $username, $extdomain, $extusername, $extpassword, $authtype) 
	{
 		//"poplnkadd"[TAB]"loc-domain"[TAB]"loc-username"[TAB]"extrn-domain"[TAB]"extrn-username"[TAB]"extrn-password"[TAB]"authtype"<CR><LF>		
 		$this->_write("poplnkadd\t".$domain."\t".$username."\t".$extdomain."\t".$extusername."\t".$extpassword."\t".$authtype);
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'AddExtLink()');
	}
	
	/**
	 * @return bool
	 */
	function DeleteExtLink($domain, $username, $extdomain, $extusername) 
	{
		//"poplnkdel"[TAB]"loc-domain"[TAB]"loc-username"[TAB]"extrn-domain"[TAB]"extrn-username"<CR><LF>
		$this->_write("poplnkdel\t".$domain."\t".$username."\t".$extdomain."\t".$extusername);
		$response = $this->_readline();
		
		return $this->_checkResponse($response, 'DeleteExtLink()');
	}
	
	/**
	 * @param string $domain
	 * @param string $alias
	 * @return bool
	 */
	function ExtLinkExist($domain, $username, $extlink)
	{
		$extlinkArray = $this->GetExtLinkList($domain, $username);
		return in_array($extlink, $extlinkArray);
	}
	
	/**
	 * @return array
	 */
	function GetExtLinkList($domain, $username)
	{
		$ext = array();
		$extList = $this->ExtLinksList($domain, $username);
		foreach ($extList as $value) {
			if ($value[1] == $username) {
				$ext[] = $value[2] . ' / ' . $value[3];
			}
		}
		return $ext;
	}
	
	/**
	 * @return array
	 */
	function ExtLinksList($domainname = null, $username = null)
	{
		//"poplnklist"[TAB]"loc-domain"[TAB]"loc-username"<CR><LF>
		$extlinkArray = array();
		if ($domainname && $username)
		{
			$this->_write("poplnklist\t".$domainname."\t".$username);	
		}
		else if ($domainname && $username == null)
		{
			$this->_write("poplnklist\t".$domainname."\t".$username);
		}
		else
		{
			$this->_write("poplnklist");	
		}
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'ExtLinksList()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tempArray = explode("\t", $response);
					$extlinkArray[] = array_map('myTrim', $tempArray);
				}
				else break;			
			} while ($response != '.');
		}
		return $extlinkArray;		
	}
	
	//==========================Mail proc================================================================
	
	/**
	 * @param string $domainname
	 * @param string $username
	 * @param array $text
	 * @return bool
	 */
	function SetMailProc($domainname, $username, $text) 
	{
		//"usersetmproc"[TAB]"domain"[TAB]"username"<CR><LF>
		//"custdomset"[TAB]"domain"<CR><LF>
		if ($username !== '' && $username !== null)
			$this->_write("usersetmproc\t".$domainname."\t".$username);
		else 
			$this->_write("custdomset\t".$domainname);
			
		$response = $this->_readline();
		
		if ($this->_checkResponse($response, 'SetMailProc();')) 
		{
			foreach ($text as $str) {
				$this->_write($str);
			}
			$this->_write("\r\n.");
			$response = $this->_readline();
			
			return $this->_checkResponse($response, 'SetMailProc();');
		} else {
			return false;
		}
	}
	
	/**
	 * @param string $domainname
	 * @param string $username
	 * @return array
	 */
	function GetMailProc($domainname, $username)
	{
 		//"usergetmproc"[TAB]"domain"[TAB]"username"<CR><LF>
 		//"custdomget"[TAB]"domain"<CR><LF>
 		$mailProc = array();
 		if ($username !== '' && $username !== null) {
			$this->_write("usergetmproc\t".$domainname."\t".$username);
 		} else { 
			$this->_write("custdomget\t".$domainname);
		}
		
		$response = $this->_readline();
		
		if ($this->_checkResponse($response, 'GetMailProc();')) 
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$mailProc[] = $response;
				}
				else break;
			} while ($response != '.');
		}
		return $mailProc;	
	}
	
	/**
	 * Finds index of command in $mailproc
	 * @param array $mailproc
	 * @param string $cmd
	 * @return int
	 */
	function MProcFindCmd($mailproc, $cmd) 
	{
		for ($i = 0; $i < count($mailproc); $i++) {
			if (substr($mailproc[$i], 1, strlen($cmd)) == $cmd) 
				return $i;
		}
		return false;
	}
	
	/**
	 * Returns list of parameters for given command
	 * @param array $mailproc
	 * @param string $cmd
	 * @return array
	 */
	function GetCmdParams($mailproc, $cmd)
	{
		$params = array();
		if (count($mailproc) > 0) {
			$index = $this->MProcFindCmd($mailproc, $cmd);
			if ($index !== false) {
				$cmdLine = $mailproc[$index];
				$tmpArray = explode("\t", $cmdLine);
				$tmpCount = count($tmpArray);
				if ($tmpCount > 1) {
					for ($i = 1; $i < $tmpCount; $i++) {
						$params[] = myTrim($tmpArray[$i]);
					}
				} else 
					return false;
			} else 
				return false;
		}
		return $params;
	}
	
	/**
	 * Checks if command already have given parameter in parameters list
	 * 
	 * @param array @mailproc
	 * @param string $cmd
	 * @param string $param
	 * @return bool
	 */
	function CmdParamExist($mailproc, $cmd, $param) 
	{
		$cmdParams = $this->GetCmdParams($mailproc, $cmd);
		if ($cmdParams !== false) 
			return in_array($param, $cmdParams);
		else 
			return false;
	}
	
	/**
	 * Adds parameter to a command in given $mailproc
	 * @param string $domain
	 * @param string $username
	 * @param string $cmd
	 * @param string $param
	 * @return bool
	 */
	function AddCmdParam($domain, $username, $cmd, $param)
	{
		$rawmailproc = $this->GetMailProc($domain, $username);
		$mailproc = array();
		$advmailproc = array();
		$this->ExplodeMailproc($rawmailproc, $mailproc, $advmailproc);
		$index = $this->MProcFindCmd($mailproc, $cmd);
		if ($index !== false) {
			$cmdLine = $mailproc[$index];
			if (!$this->CmdParamExist($mailproc, $cmd, $param)) 
			{
				$cmdLine .= "\t\"" . $param . '"';
				$mailproc[$index] = $cmdLine;
				$rawmailproc = $this->ImplodeMailproc($mailproc, $advmailproc);
				return $this->SetMailProc($domain, $username, $rawmailproc);
			} 
			else {
				$this->_error = "Add $cmd ('$param') - already exist";
				return false;
			}
		} else {
			$cmdLine = myUnTrim($cmd) . "\t" . myUnTrim($param);
			$mailproc[] = $cmdLine;
			$rawmailproc = $this->ImplodeMailproc($mailproc, $advmailproc);
			return $this->SetMailProc($domain, $username, $rawmailproc);
		}
	}
	
	/**
	 * @return bool
	 */
	function DeleteCmdParam($domain, $username, $cmd, $param) 
	{
		$rawmailproc = $this->GetMailProc($domain, $username);
		$mailproc = $advmailproc = '';
		$this->ExplodeMailproc($rawmailproc, $mailproc, $advmailproc);
		$index = $this->MProcFindCmd($mailproc, $cmd);
		if ($index !== false) {
			if ($this->CmdParamExist($mailproc, $cmd, $param)) 
			{
				$params = $this->GetCmdParams($mailproc, $cmd);
				if (count($params) > 1) {
					$newCmd = '"' . $cmd . '"';
					foreach ($params as $tmp) {
						if ($tmp != $param) 
							$newCmd .= "\t\"" . $tmp . "\"";
					}
					$mailproc[$index] = $newCmd;
				} elseif (count($params) == 1) {
					array_splice($mailproc, $index, 1);
				}
				
				if (count($advmailproc) > 0)
					$resultmailproc = $this->ImplodeMailproc($mailproc, $advmailproc);
				else 
					$resultmailproc = $mailproc;
				return $this->SetMailProc($domain, $username, $resultmailproc);
			} else {
				$this->_error = "There is no Delete$cmd ('$param')!";
				return false;
			}
		}
		return false;
	}
	
	/**
	 * Explodes Mailproc array in two pieces: main and advanced
	 * 
	 * @param array $mailproc
	 * @param array $userMailproc
	 * @param array $advMailproc
	 * @return bool
	 */
	function ExplodeMailproc($mailproc, &$userMailproc, &$advMailproc) 
	{
		if (count($mailproc) > 0) {
			foreach ($mailproc as $str) {
				$t = explode("\t", $str);
				if (!in_array(myTrim($t[0]), array('mailbox', 'redirect', 'lredirect'))) {
					$advMailproc[] = $str;
				} else {
					$userMailproc[] = $str;
				}
			}
		} else {
			return false;
		}
		return true;
	}
	
	/**
	 * Implodes two parts of Mailproc and return a result array
	 *
	 * @param array $userMailproc
	 * @param array $advMailproc
	 * @return array
	 */
	function ImplodeMailproc($userMailproc, $advMailproc) 
	{
		return array_merge($userMailproc, $advMailproc);
	}
	
	//============================SERVER Tab Functions===================================================

	/**
	 * Return parameter value
	 *
	 * @param array $mailproc
	 * @param string  $cmd
	 * @return string 
	 */
	function GetCmdParamValue($mailproc, $cmd)
	{
		$param = '';
		if (count($mailproc) > 0) {
			$index = $this->MProcFindCmd($mailproc, $cmd);
			if ($index !== false) {
				$cmdLine = $mailproc[$index];
				$tmpArray = explode("\t", $cmdLine);
				$tmpCount = count($tmpArray);
				if ($tmpCount > 1) {
					$param = $tmpArray[1];
					return $param;
				} else 
					return false;
			} else {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Set parameter value
	 *
	 * @param array $mailproc
	 * @param string  $cmd
	 * @return array 
	 */
	function SetCmdParamValue($mailproc, $cmd, $value) 
	{
		if (count($mailproc) > 0) {
			$index = $this->MProcFindCmd($mailproc, $cmd);
			if ($index !== false) {
				$cmdLine = $mailproc[$index];
				$tmpArray = explode("\t", $cmdLine);
				$tmpCount = count($tmpArray);
				if ($tmpCount > 1) {
					$tmpArray[1] = '"' . $value . '"';
					$mailproc[$index] = implode("\t", $tmpArray);
					return $mailproc;
				} else 
					return false;
			} else {
				return false;
			}
		}
		return true;
	}
	
	//===================================================================================================
	//====================================SETTINGS=======================================================
	//===================================================================================================
	
	/**
	 * @param string $file
	 * @return array
	 */
	function GetTabFile($file)
	{
		//"cfgfileget"[TAB]"relative-file-path"<CR><LF>
		$tab = array();
		$this->_write("cfgfileget\t$file");	
		$resp = trim($this->_readline());
		if ($this->_checkResponse($resp, 'GetTabFile()'))
		{
			do 
			{
				$response = trim($this->_readline());
				if ($response != '.')
				{
					$tab[] = $response;
				}
				else break;			
			} while ($response != '.');
		}
		return $tab;	
	}
	
	/**
	 * @param string $file
	 * @param array $tab
	 */
	function SetTabFile($file, $tab)
	{
		//"cfgfileset"[TAB]"relative-file-path"<CR><LF>
		$this->_write("cfgfileset\t$file");
		$response = $this->_readline();
		if ($this->_checkResponse($response, 'SetTabFile();')) 
		{
			foreach ($tab as $tt) {
				$tt = trim($tt);
				if (strlen($tt) > 0)
				{
					$this->_write($tt);
				}
			}
			
			$this->_write('');
			$this->_write('.');
			$response = $this->_readline();
			
			return $this->_checkResponse($response, 'SetTabFile();');
		} else {
			return false;
		}
	}
	
	/**
	 *
	 * @return array
	 */
	function GetCtrlAccounts($accTab)
	{
		$accounts = array();
		foreach ($accTab as $acc) {
			$tmpArr = explode("\t", $acc);
			$accounts[] = myTrim($tmpArr[0]);
		}
		return $accounts;
	}
	
	/**
	 *
	 * @param array $ctrlAccounts
	 * @param string$account
	 * @return bool
	 */
	function CtrlAccountExist($accTab, $name) 
	{
		foreach ($accTab as $acc) {
			$tmpArr = explode("\t", $acc);
			if (myTrim($tmpArr[0]) == $name) 
				return true;
		}
		return false;
	}
	
	/**
	 * @param string $name
	 * @param string $password
	 * @return bool
	 */
	function AddCtrlAccount($name, $password) 
	{
		$accTab = $this->GetTabFile('ctrlaccounts.tab');
		$accTab[] = myUnTrim($name) . "\t" . EncryptPassword($password);
		return $this->SetTabFile('ctrlaccounts.tab', $accTab);
	}
	
	/**
	 * @param string $name
	 * @return bool
	 */
	function DeleteCtrlAccount($name)
	{
		$accTab = $this->GetTabFile('ctrlaccounts.tab');
		if ($accTab) 
		{
			for ($i = 0; $i < count($accTab); $i++) 
			{
				$acc = $accTab[$i];
				$tmpArr = explode("\t", $acc);
				if (myTrim($tmpArr[0]) == $name) 
					array_splice($accTab, $i, 1);
			}
		} else 
			return false;
		if (!$this->SetTabFile('ctrlaccounts.tab', $accTab)) return false;
		return true;
	}
	
	/**
	 * Prints setting value
	 *
	 * @param string $name
	 */
	function PrintServerSettingValue($name) 
	{
		print isset($this->_settings[$name][0]) ? str_replace('"', '&quot;', $this->_settings[$name][0]) : '';
	}
	
	/**
	 * Prints that setting checked or not
	 *
	 * @param string $name
	 */
	function PrintServerSettingCheck($name)
	{
		global $ServerSettingBoxes;
		global $ServerSettingsWithInvertion;
		
		if (in_array($name, $ServerSettingBoxes)) 
		{
			if (isset($this->_settings[$name][0]))
			{
				if (isset($this->_settings[$name][1]) && $this->_settings[$name][1] == '#')
				{
				}
				else
				{
					if (in_array($name, $ServerSettingsWithInvertion)) 
					{
						print ($this->_settings[$name][0] == '0') ? 'checked' : '';
					}
					else
					{
						print ($this->_settings[$name][0] == '1') ? 'checked' : '';
					}
				}
			}
		}
		else 
		{
			if (isset($this->_settings[$name][1]))
			{
				print ($this->_settings[$name][1] == '#') ? '' : 'checked';
			}
		}
	}
	
	/**
	 * Write server.tab
	 *
	 * @param array $tab
	 * @return bool
	 */
	function SetServerSettings($tab)
	{
		return $this->SetTabFile('server.tab', $tab);
	}
	
	//===========================LOGS====================================================================
	
	/**
	 * Checks log file for existing
	 *
	 * @param string $prefix (see $PosibleLognames for aviable names)
	 * @return bool
	 */
	function isLogExist($prefix) 
	{
		$date = getdate();
		$lastLogName = $this->GetLastLogName($prefix . '-' . $date['year'] . '*');
		if ($lastLogName !== false) 
			return true;
		else  
			return false;
	}
	
	/**
	 * Return filename of last written log file
	 *
	 * @param strting $queryString
	 * @return string
	 */
	function GetLastLogName($queryString)
	{
		//"filelist"[TAB]"relative-dir-path"[TAB]"match-string"<CR><LF>
		$filelist = array();
		$this->_write("filelist\tlogs\t".$queryString);
		$response = $this->_readline();
		if ($this->_checkResponse($response, 'GetLastLogName();')) 
		{
			do 
				{
					$response = trim($this->_readline());
					if ($response != '.')
					{
						$filelist[] = $response;
					}
					else break;			
				} while ($response != '.');
		}
		if (count($filelist) != 0) {
			$logname = explode("\t", array_pop($filelist));
			return myTrim($logname[0]); // zero element is a filename
		}
		return false;	
	}
	
	/**
	 * @param string $prefix(see $PosibleLognames for aviable names)
	 */
	function GetLog($prefix)
	{
		$logg = array();
		$date = getdate();
		//"cfgfileget"[TAB]"logs/log-TIMESTAMP"<CR><LF>
		if ($this->isLogExist($prefix)) {
			$lastLogName = $this->GetLastLogName($prefix . '-' . $date['year'] . '*');
			$this->_write("cfgfileget\tlogs/$lastLogName");
			$response = $this->_readline();
			if ($this->_checkResponse($response, 'GetLog();')) 
			{
				do 
				{
					$response = trim($this->_readline());
					if ($response != '.')
					{
						$logg[] = $response;
					}
					else break;			
				} while ($response != '.');
				
				return $logg; 
				
			} else 
				return false;
		} else 
			return false;
	}
	
	//===================================================================================================
	
	function _cleanup()
	{
		if (is_resource($this->_socket))
		{
		    socket_set_blocking($this->_socket, false);
		    @fclose($this->_socket);
		    $this->_socket = false;
		}
	}

	/**
	 * @param string $string
	 * @return bool
	 */
	function _write($string)
	{
		$start = ap_Utils::Microtime();
		CAdminPanel::Log('XM'.(++$GLOBALS[AP_WM_COUNT]).' > '.$string);
		if(!@fwrite($this->_socket , $string."\r\n" , strlen($string."\r\n")))
		{
			$GLOBALS[AP_WM_TIME] += (ap_Utils::Microtime() - $start);
			$this->_error = '_write() - Error while send "'.$string.'". -- Connection closed.';
			$this->_clear_error = 'Error while send "'.$string.'".';
			$this->_cleanup();
			return false;
		}
		$GLOBALS[AP_WM_TIME] += (ap_Utils::Microtime() - $start);
		
		return true;
	}
	
	function _readline($buffer_size = 512)
	{
		$start = ap_Utils::Microtime();
		$buffer = @fgets($this->_socket, $buffer_size);
		CAdminPanel::Log('XM < '.trim($buffer));
		$GLOBALS[AP_WM_TIME] += (ap_Utils::Microtime() - $start);
		
		$this->socket_status = @socket_get_status($this->_socket);
		if($this->socket_status["timed_out"])
		{
		    $this->_cleanup();
		    return false;
		}
		$this->socket_status = false;
		return $buffer;
	}
	
	/**
	 * @return string
	 */
	function GetError()
	{
		return $this->_error;
	}

	/**
	 * @return string
	 */
	function GetClearError()
	{
		if ((strlen($this->_clear_error) > 0))
		{
			if ($this->_clear_error{0} == '-')
			{
				$start = strpos($this->_clear_error, ' ');
				return ($start !== false) ? substr($this->_clear_error, $start + 1) : $this->_clear_error;
			}
		}
		return $this->GetError();
	}
	
	/**
	 * @param string $response
	 * @param string $functionName
	 * @return bool
	 */
	function _checkResponse($response, $functionName)
	{
		if (strlen($response) < 1) 
		{
			$this->_error = $functionName.' - Error: response is null';
			$this->_clear_error = 'Response is null';
			$this->_cleanup();
			CAdminPanel::Log('XM ERROR < '.trim($this->_error));
			return false;
		}
		if (substr($response,0,1) == '-' )
		{
			$this->_error = $functionName.' - Error: '.$response;
			$this->_clear_error = $response;
			CAdminPanel::Log('XM ERROR < '.trim($this->_error));
			return false;
		}
		else if (substr($response,0,1) == '+' )
		{
			return true;
		}
		else 
		{
			$this->_error = $functionName.' - Unknown Error: '.$response;
			$this->_clear_error = $response;
			CAdminPanel::Log('XM ERROR < '.trim($this->_error));
			$this->_cleanup();
			return false;
		}
	}
}

	function myTrim($string)
	{
		return trim($string, '"');
	}
	
	/**
	 * Just need it
	 *
	 * @param string $string
	 * @return string
	 */
	function myUnTrim($string)
	{
		return '"' . $string . '"';
	}
	
?>